@@ -11,7 +11,12 @@ from os import listdir
 from os.path import isfile, join
 import json
 import uuid
+import urllib.request
+from urllib.request import urlopen
+from urllib.request import urlretrieve
+import cgi
 import requests
+import tarfile
 from geopandas import GeoSeries
 from shapely.geometry import Polygon
 import geopandas as gpd
@@ -20,7 +25,6 @@ from shapely.geometry import Polygon
 from vallaris.utils import *
 from IPython.display import display, Javascript, Markdown as md, HTML
 from urllib.parse import urlencode
-import urllib.request
 import zipfile
 from IPython.display import IFrame
 import rasterio
@@ -31,10 +35,10 @@ import matplotlib.pyplot as plt
 import earthpy as et
 import earthpy.spatial as es
 import earthpy.plot as ep
-from urllib.request import urlretrieve
 from dotenv import load_dotenv
 load_dotenv()
 
+
 def setEnviron(parameter, *args, **kwargs):
     storage = kwargs.get('storage', False)
 
@@ -428,7 +432,7 @@ def DeleteCollection(parameter, *args, **kwargs):
         return deleteData
 
 
-def CreateTile(parameter, *args, **kwargs):
+def CreateVectorTile(parameter, *args, **kwargs):
     storage = kwargs.get('storage', False)
     try:
         msgBody = json.loads(parameter)["requestBody"]
@@ -502,46 +506,46 @@ def ProcessFail(storage, parameter, msg):
     print("process failed")
     return json.dumps(out_msg)
 
+
 def RenderFeatures(parameter, *args, **kwargs):
     opacity = kwargs.get('opacity', 0.4)
     color = kwargs.get('color', "#FF0000")
     HOST = os.environ["VallarisServer"]
-    
-    
-    try :
-    
-        data_out2 =[]
-        try :
+
+    try:
+
+        data_out2 = []
+        try:
             geojson = parameter['data']
             geojson2 = geojson['geometry'].to_json()
             dump_json = json.loads(geojson2)
             data_ = {
-                    "id":str(parameter['name']),
-                    "data":dump_json,
-                    "layer_type":[parameter['layer_type']],
-                "paint":{"opacity":opacity,"color" : color},
-                }
+                "id": str(parameter['name']),
+                "data": dump_json,
+                "layer_type": [parameter['layer_type']],
+                "paint": {"opacity": opacity, "color": color},
+            }
             data_out2.append(data_)
 
-        except :
+        except:
             crt = len(parameter)
-            for i in range(0,crt):
+            for i in range(0, crt):
                 geojson = parameter[i]['data']
                 geojson2 = geojson['geometry'].to_json()
                 dump_json = json.loads(geojson2)
                 data_ = {
-                    "id":str(parameter[i]['name']),
-                    "data":dump_json,
-                    "layer_type":[parameter[i]['layer_type']],
-                    "paint":{"opacity":opacity,"color" : color},
-                    }
+                    "id": str(parameter[i]['name']),
+                    "data": dump_json,
+                    "layer_type": [parameter[i]['layer_type']],
+                    "paint": {"opacity": opacity, "color": color},
+                }
                 data_out2.append(data_)
 
-        parameter2 = {"features":data_out2}
-        
+        parameter2 = {"features": data_out2}
+
         map_id = uuid.uuid4().hex
         map_id2 = '"'
-        
+
         js_create = """
                 <iframe id='"""+map_id+"""' " width="100%" height=336 src=""></iframe>
                 <script>
@@ -556,74 +560,74 @@ def RenderFeatures(parameter, *args, **kwargs):
                 </script>
             """
 
-        def js_convert_str_html(p,url):
-            js_convert = js_create.format(p,url)
+        def js_convert_str_html(p, url):
+            js_convert = js_create.format(p, url)
             return HTML(js_convert)
-        
+
         newurl = HOST + "/sandbox/render/iframe"
-        mydict = {'params': parameter2 , 'z':5,'p':0,}
+        mydict = {'params': parameter2, 'z': 5, 'p': 0, }
 
-
-        jsobj = js_convert_str_html(json.dumps(mydict),newurl)
+        jsobj = js_convert_str_html(json.dumps(mydict), newurl)
         # display(jsobj)
-        
+
         return display(jsobj)
-    
+
     except Exception as e:
         return 'error display :' + str(e)
 
+
 def RenderCollection(parameter, *args, **kwargs):
     opacity = kwargs.get('opacity', 0.4)
     color = kwargs.get('color', "#FF0000")
-    
-    try :
+
+    try:
         HOST = os.environ["VallarisServer"]
-        data_out2 =[]
-        try :
+        data_out2 = []
+        try:
             if parameter['layer_type'] == 'clusterPoint':
                 _type = ['clusterPoint', 'clusterPointCount']
-            else :
+            else:
                 _type = [parameter['layer_type']]
+            
+            Api_Key = os.environ["APIKey"]
 
-            data_ =  {"id":parameter['data'], 
-                 "api_key": parameter['api_key'],
-                      "paint":
-                     {"opacity": opacity,
-                      "color" : color
-                     },
-                     "layer_type":[parameter['layer_type']]
+            data_ = {"id": parameter['data'],
+                     "api_key": Api_Key,
+                     "paint":
+                     {"opacity": float(opacity),
+                      "color": color
+                      },
+                     "layer_type": [parameter['layer_type']]
 
-                }
+                     }
             data_out2.append(data_)
 
-        except :
+        except:
             crt = len(parameter)
-            for i in range(0,crt):
+            for i in range(0, crt):
                 if parameter[i]['layer_type'] == 'clusterPoint':
                     _type = ['clusterPoint', 'clusterPointCount']
-                else :
+                else:
                     _type = [parameter[i]['layer_type']]
 
-                data_ = {"id":parameter[i]['data'], 
-                     "api_key": parameter[i]['api_key'],
-                     "paint":
-                     {"opacity": opacity,
-                      "color" : color
-                     },
-                     "layer_type":_type
-                    }
+                data_ = {"id": parameter[i]['data'],
+                         "api_key": Api_Key,
+                         "paint":
+                         {"opacity": float(opacity),
+                          "color": color
+                          },
+                         "layer_type": _type
+                         }
 
                 data_out2.append(data_)
 
-
-        data_items = {"collection":data_out2}
-
+        data_items = {"collection": data_out2}
 
         params = json.dumps(data_items)
 
-        mydict = {'params': params,'z':3,'p':0}
+        mydict = {'params': params, 'z': 3, 'p': 0, 'host': HOST}
 
-        qstr = urlencode(mydict)   
+        qstr = urlencode(mydict)
 
         mapURL = '/sandbox/render/iframe?'
 
@@ -632,39 +636,39 @@ def RenderCollection(parameter, *args, **kwargs):
         out_iframe = IFrame(url, '100%', '336px')
 
         return out_iframe
-    
+
     except Exception as e:
         return 'error display :' + str(e)
 
-    
 
 def RenderRaster(parameter, *args, **kwargs):
     opacity = kwargs.get('opacity', 1)
     option = kwargs.get('option', 'internal')
-    
+
     if option == 'internal':
-        url = "http://172.30.10.126:7000/" + parameter['data']
-    
-    else :
+        SandboxFileserver = os.environ["SandboxFileserver"]
+        LOGNAME = os.environ["LOGNAME"]
+        url = SandboxFileserver + "/" + str(LOGNAME) + "/" + parameter['data']
+
+    else:
         url = parameter['data']
-        
-    
-    try :
+
+    try:
         HOST = os.environ["VallarisServer"]
         data_items = {
-           "coverage":[
-              {
-                 "id":"rasterHot",
-                 "url": url,
-                 "layer_name": parameter['layer_type'],
-                 "VallarisServer": HOST,
-                 "paint":{
-                    "opacity": opacity
-                 }
-              }
-           ]
+            "coverage": [
+                {
+                    "id": "rasterHot",
+                    "url": url,
+                    "layer_name": parameter['layer_type'],
+                    "VallarisServer": HOST,
+                    "paint":{
+                        "opacity": float(opacity)
+                    }
+                }
+            ]
         }
-        
+
         map_id = uuid.uuid4().hex
         map_id2 = '"'
 
@@ -681,32 +685,39 @@ def RenderRaster(parameter, *args, **kwargs):
                 bake_storage(myinputstring)
                 </script>
             """
-        def js_convert_str_html_raster(p,url):
-            js_convert = js_create.format(p,url)
+
+        def js_convert_str_html_raster(p, url):
+            js_convert = js_create.format(p, url)
             return HTML(js_convert)
 
-
-        mydict = {'params': data_items , 'z':8,'p':0,}
+        mydict = {'params': data_items, 'z': 8, 'p': 0, }
 
         newurl = HOST + "/sandbox/render/iframe"
 
-        jsobj = js_convert_str_html_raster(json.dumps(mydict),newurl)
+        jsobj = js_convert_str_html_raster(json.dumps(mydict), newurl)
         # display(jsobj)
 
         return display(jsobj)
-    
-    
+
     except Exception as e:
         return 'error display :' + str(e)
 
+
 def RGBMap(parameter, *args, **kwargs):
-    
+
     title = kwargs.get('title', "Maps")
     ylabel = kwargs.get('ylabel', "Row #")
     xlabel = kwargs.get('xlabel', "Column #")
-    
-    try :
-    
+
+    try:
+
+        def normalize(array):
+            '''
+            normalize: normalize a numpy array so all value are between 0 and 1
+            '''
+            array_min, array_max = array.min(), array.max()
+            return (array - array_min) / (array_max - array_min)
+
         # Open the file:
         raster = rasterio.open(parameter['data'])
 
@@ -714,57 +725,63 @@ def RGBMap(parameter, *args, **kwargs):
         b2 = raster.read(parameter['band'][1])
         b3 = raster.read(parameter['band'][2])
 
-        nrg = np.dstack((b1, b2, b3))
+        b1_norm = normalize(b1)
+        b2_norm = normalize(b2)
+        b3_norm = normalize(b3)
 
-        fig, ax = plt.subplots(figsize = (12, 12))
+        nrg = np.dstack((b1_norm, b2_norm, b3_norm))
+
+        fig, ax = plt.subplots(figsize=(12, 12))
         im = ax.imshow(nrg.squeeze())
         # ep.colorbar(im)
-        ax.set(title= title)
-        ax.set(xlabel= xlabel +'\n'+'\n'+ '© Vallaris Maps')
-        ax.set(ylabel= ylabel)
+        ax.set(title=title)
+        ax.set(xlabel=xlabel + '\n'+'\n' + '© Vallaris Maps')
+        ax.set(ylabel=ylabel)
 
         return plt.show()
-    
+
     except Exception as e:
         return 'error display :' + str(e)
 
+
 def NDVIMap(parameter, *args, **kwargs):
-    
+
     title = kwargs.get('title', "NDVI")
     ylabel = kwargs.get('ylabel', "Row #")
     xlabel = kwargs.get('xlabel', "Column #")
-    
-    try :
-    
+
+    try:
+
         # Open the file:
         raster = rasterio.open(parameter['data'])
-        
+
         red = raster.read(parameter['red'])
         nir = raster.read(parameter['nir'])
         np.seterr(divide='ignore', invalid='ignore')
         c_map = kwargs.get('cmap', 'viridis')
-        minn = kwargs.get('min', -1) 
-        maxx = kwargs.get('max', 1) 
-        
-        # Calculate ndvi
-        ndvi = (nir.astype(float)-red.astype(float))/(nir.astype(float)+red.astype(float))
+        minn = kwargs.get('min', -1)
+        maxx = kwargs.get('max', 1)
 
-        fig, ax = plt.subplots(figsize = (12, 12))
+        # Calculate ndvi
+        ndvi = (nir.astype(float)-red.astype(float)) / \
+            (nir.astype(float)+red.astype(float))
+
+        fig, ax = plt.subplots(figsize=(12, 12))
         im = ax.imshow(ndvi.squeeze(), cmap=c_map, vmin=minn, vmax=maxx)
         ep.colorbar(im)
-        ax.set(title= title)
-        ax.set(xlabel= xlabel +'\n'+'\n'+ '© Vallaris Maps')
-        ax.set(ylabel= ylabel)
-        
+        ax.set(title=title)
+        ax.set(xlabel=xlabel + '\n'+'\n' + '© Vallaris Maps')
+        ax.set(ylabel=ylabel)
+
         return plt.show()
-    
+
     except Exception as e:
         return 'error display :' + str(e)
 
 
 def RGBSave(parameter, *args, **kwargs):
-    
-    try : 
+
+    try:
         nodata = kwargs.get('nodata', -99)
         dtype = kwargs.get('dtype', "int8")
 
@@ -779,25 +796,25 @@ def RGBSave(parameter, *args, **kwargs):
         nrg = np.dstack((b1, b2, b3))
 
         ras_meta = raster.profile
-        ras_meta['count']= 3
+        ras_meta['count'] = 3
         ras_meta['dtype'] = dtype
         ras_meta['nodata'] = nodata
 
         with rasterio.open(parameter['output'], 'w', **ras_meta) as dst:
             # If array is in (x, y, z) order (cols, rows, bands)
             dst.write(np.moveaxis(nrg, [0, 1, 2], [1, 2, 0]))
-            
+
         return 'Succeed : ' + parameter['output']
-            
-    
+
     except Exception as e:
         return 'error to save :' + str(e)
 
+
 def NDVISave(parameter, *args, **kwargs):
     nodata = kwargs.get('nodata', -99)
     dtype = kwargs.get('dtype', "float32")
-    
-    try :
+
+    try:
 
         # Open the file:
         raster = rasterio.open(parameter['input'])
@@ -807,22 +824,23 @@ def NDVISave(parameter, *args, **kwargs):
         np.seterr(divide='ignore', invalid='ignore')
 
         # Calculate ndvi
-        ndvi = (nir.astype(float)-red.astype(float))/(nir.astype(float)+red.astype(float))
+        ndvi = (nir.astype(float)-red.astype(float)) / \
+            (nir.astype(float)+red.astype(float))
 
         ras_meta = raster.profile
-        ras_meta['count']=1
+        ras_meta['count'] = 1
         ras_meta['dtype'] = "float32"
         ras_meta['nodata'] = -99
 
         with rasterio.open(parameter['output'], 'w', **ras_meta) as dst:
             dst.write(ndvi, 1)
-        
+
         return 'Succeed : ' + parameter['output']
-            
-    
+
     except Exception as e:
         return 'error to save :' + str(e)
 
+
 def GrayMap(parameter, *args, **kwargs):
 
     title = kwargs.get('title', "Maps")
@@ -831,33 +849,33 @@ def GrayMap(parameter, *args, **kwargs):
     minn = kwargs.get('min', "0")
     maxx = kwargs.get('max', "255")
     c_map = kwargs.get('cmap', "gray")
-    
-    
-    try :
-    
+
+    try:
+
         # Open the file:
         raster = rasterio.open(parameter['data'])
 
         b1 = raster.read(parameter['band'])
 
-        fig, ax = plt.subplots(figsize = (12, 12))
+        fig, ax = plt.subplots(figsize=(12, 12))
         im = ax.imshow(b1.squeeze(), cmap=c_map, vmin=minn, vmax=maxx)
         ep.colorbar(im)
-        ax.set(title= title)
-        ax.set(xlabel= xlabel +'\n'+'\n'+ '© Vallaris Maps')
-        ax.set(ylabel= ylabel)
+        ax.set(title=title)
+        ax.set(xlabel=xlabel + '\n'+'\n' + '© Vallaris Maps')
+        ax.set(ylabel=ylabel)
 
         return plt.show()
-    
+
     except Exception as e:
         # print(e)
         return 'error display :' + str(e)
 
+
 def GraySave(parameter, *args, **kwargs):
     nodata = kwargs.get('nodata', -99)
     dtype = kwargs.get('dtype', "float32")
-    
-    try :
+
+    try:
 
         # Open the file:
         raster = rasterio.open(parameter['input'])
@@ -865,150 +883,535 @@ def GraySave(parameter, *args, **kwargs):
         image = raster.read(parameter['band'])
 
         ras_meta = raster.profile
-        ras_meta['count']=1
+        ras_meta['count'] = 1
         ras_meta['dtype'] = "float32"
         ras_meta['nodata'] = -99
 
         with rasterio.open(parameter['output'], 'w', **ras_meta) as dst:
-            dst.write(image,1)
-        
+            dst.write(image, 1)
+
         return 'Succeed : ' + parameter['output']
-            
-    
+
     except Exception as e:
         print(e)
         return 'error to save : please check parameter'
 
 
-def SearchData(key_word, *args, **kwargs):
-    
+def SearchData(satellite, *args, **kwargs):
+    offset = kwargs.get('offset', 0)
+    bbox = kwargs.get('bbox', False)
+    datetime = kwargs.get('datetime', False)
+    cloud_cover = kwargs.get('cloud_cover', False)
+    path = kwargs.get('path', False)
+    row = kwargs.get('row', False)
+    relative_orbit_number = kwargs.get('relative_orbit_number', False)
+    tile_number_field = kwargs.get('tile_number_field', False)
+
     try:
 
-        APIKey = '277E01W1DI8YhDYYGre10pi8CTm5N3bQfPx4VX0hRVnjYOuoFIrmmWM1nY6GKVGL'
-
-        url = "https://v2k-dev.vallarismaps.com/core/api/features/1.0/collections?search="+key_word+"&itemType=Coverage"
-
-        payload = {}
-        headers = {
-            'API-Key': APIKey,
+        payload = {
+            "sortby": "-id",
+            "limit": 10,
+            "offset": offset
         }
 
-        response = requests.request("GET", url, headers=headers, data=payload)
-        # print(response)
+        if bbox != False:
+            payload['bbox'] = bbox
 
-        json_col = response.json()
-        # print(json_col)
+        if datetime != False:
+            payload['datetime'] = datetime
 
-        crt = len(json_col['collections'])
-        # print(crt)
+        if satellite == 'Landsat8' or satellite == 'Landsat8':
 
-        outdata = []
-        for c in range(0, crt):
-            name = json_col['collections'][c]['title']
-        #     getName.append(name)
-            col = json_col['collections'][c]['id']
-        #     getCol.append(col)
+            option = []
+            if cloud_cover != False:
 
-            jsonout ={
-                "id": col,
-                "title" : name
-            }
+                cloud_ = {
+                    "op": "<=",
+                    "args": [
+                        {
+                            "property": "eo:cloud_cover"
+                        },
+                        cloud_cover
+                    ]
+                }
+                option.append(cloud_)
 
-            outdata.append(jsonout)
+            if path != False:
 
-        return outdata
-    
-    
-    except Exception as e:
-        return 'error :' + str(e)
+                path_ = {
+                    "op": "=",
+                    "args": [
+                        {
+                            "property": "landsat:wrs_path"
+                        },
+                        int(path)
+                    ]
+                }
+                option.append(path_)
 
-def ExportCoverage(collection):
-    
-    try :
+            if row != False:
 
-        APIKey = "277E01W1DI8YhDYYGre10pi8CTm5N3bQfPx4VX0hRVnjYOuoFIrmmWM1nY6GKVGL"
+                row_ = {
+                    "op": "=",
+                    "args": [
+                        {
+                            "property": "landsat:wrs_row"
+                        },
+                        int(row)
+                    ]
+                }
+                option.append(row_)
 
-        url = "https://v2k-dev.vallarismaps.com/core/api/coverages/1.0-beta/manager/export/" + str(collection) + '/jobs'
+            if cloud_cover != False or path != False or row != False:
+                payload['filter'] = {"op": "and",
+                                     "args": option
+                                     }
 
+        if satellite == 'Sentinel2':
+            option = []
 
-        payload = json.dumps({
-          "option": {
-            "mimeType": "original"
-          }
-        })
+            if cloud_cover != False:
+
+                cloud_ = {
+                    "op": "<=",
+                    "args": [
+                        {
+                            "property": "cloud_coverage"
+                        },
+                        cloud_cover
+                    ]
+                }
+                option.append(cloud_)
+
+            if relative_orbit_number != False:
+
+                relative_orbit_number_ = {
+                    "op": "=",
+                    "args": [
+                        {
+                            "property": "relative_orbit_number"
+                        },
+                        relative_orbit_number
+                    ]
+                }
+                option.append(relative_orbit_number_)
+
+            if tile_number_field != False:
+
+                tile_number_field_ = {
+                    "op": "=",
+                    "args": [
+                        {
+                            "property": "tile_number_field"
+                        },
+                        tile_number_field
+                    ]
+                }
+                option.append(tile_number_field_)
+
+            if cloud_cover != False or relative_orbit_number != False or tile_number_field != False:
+                payload['filter'] = {"op": "and",
+                                     "args": option
+                                     }
+
+        # print(payload)
+        HOST = os.environ["VallarisServer"]
+        url = HOST + "/core/api/stac/1.0/"+satellite+"/search"
+
+        payload = json.dumps(payload)
         headers = {
-          'API-Key': APIKey,
-          'Content-Type': 'application/json'
+            'Content-Type': 'application/json'
         }
 
         response = requests.request("POST", url, headers=headers, data=payload)
 
-        response2 = response.json()
+        # print(response.text)
 
-        # print(response2)
+        res_json = response.json()
+        crt_res_json = len(res_json['features'])
+
+        results = []
+
+        for i in range(0, crt_res_json):
+            results.append(res_json['features'][i]['id'])
+
+        json_results = {
+            "data": results,
+            "context": res_json['context']
+        }
+
+        return json_results
+
+    except Exception as e:
+        return 'error :' + str(e)
 
 
-        jobs = response2['jobID']
+def GetItem(satellite, collection):
 
+    try:
+        HOST = os.environ["VallarisServer"]
+        url = HOST + "/core/api/stac/1.0/"+satellite + \
+            "/search?collections=" + str(collection)
+        payload = {}
+        headers = {}
+
+        response = requests.request("GET", url, headers=headers, data=payload)
+        res_json = response.json()
+
+        return res_json['features']
+
+    except Exception as e:
+        return 'error :' + str(e)
+
+
+def ExportCoverage(satellite, collection):
+
+    try:
+        Api_Key = os.environ["APIKey"]
+        print("start perform download")
+
+        HOST = os.environ["VallarisServer"]
+
+        url = HOST + "/core/api/stac/1.0/"+satellite + \
+            "/search?collections=" + str(collection)
+
+        payload = {}
+        headers = {}
+
+        response = requests.request("GET", url, headers=headers, data=payload)
+        res_json = response.json()
+
+        url_dl = res_json['features'][0]['assets']['download']['href']
+
+        url = url_dl + "?api_key=" + str(Api_Key)
+
+        print("start download")
+
+        remotefile = urlopen(url)
+        blah = remotefile.info()['Content-Disposition']
+        value, params = cgi.parse_header(blah)
+        filename = params["filename"]
+        urlretrieve(url, filename)
+        print("running download : " + filename)
+
+        fileURL = filename.split(".")[-1]
+
+        print("running download : " + filename)
+
+        if fileURL.split(".")[-1] == 'zip':
+            newpath = filename
+            new_folder = newpath.split(".")[0]
+
+            if not os.path.exists(newpath):
+                os.makedirs(new_folder)
+
+            with zipfile.ZipFile(newpath, 'r') as zip_ref:
+                zip_ref.extractall(new_folder)
+
+        elif fileURL.split(".")[-1] == 'tar' or fileURL.split(".")[-1] == 'gz':
+            newpath = filename
+            new_folder = newpath.split(".")[0]
+
+            if not os.path.exists(newpath):
+                os.makedirs(new_folder)
+            # open file
+            file = tarfile.open(newpath)
+
+            # extracting file
+            file.extractall(new_folder)
+            file.close()
+
+        else:
+            pass
+            return "error : fiile not support"
+
+        return "succeed store in folder : " + new_folder
+
+    except Exception as e:
+        return 'error :' + str(e)
+
+
+def BandStack(parameter):
+
+    try:
+        stack_band_paths = parameter['input']
+        raster_out_path = parameter['output']
+
+        stack_band_paths.sort()
+        # Create image stack and apply nodata value for Landsat
+        arr_st, meta = es.stack(
+            stack_band_paths, out_path=raster_out_path)
+
+        return 'Succeed : ' + parameter['output']
+
+    except Exception as e:
+        print(e)
+        return 'error to save : please check parameter'
+
+
+def CreateCoverage(parameter):
+
+    try:
+        Api_Key = os.environ["APIKey"]
+        file_in = parameter['data']
+        file_get = file_in.split("/")[-1]
+
+        shutil.copy(file_in, file_get)
+
+        zip_id = uuid.uuid4().hex
+        file_out = zip_id + ".zip"
+
+        with zipfile.ZipFile(file_out, mode="w") as archive:
+            archive.write(file_get)
+
+        print("running create zipfile : " + file_out)
+
+        HOST = os.environ["VallarisServer"]
+
+        url = HOST + "/core/api/utilities/1.0/files"
+        payload = {'authRequired': 'true',
+                   'expiresIn': '1d1h1m1s'
+                   }
+        headers = {
+            'API-Key': Api_Key
+        }
+
+        files = [('files', (file_out, open(file_out, 'rb'), 'application/zip'))]
+
+        response = requests.request(
+            "POST", url, headers=headers, data=payload, files=files)
+
+        # print(response.json())
+        data = response.json()
+        file_import = data['files'][0]['id']
+        collection = parameter['id']
+
+        print("running import coverage")
+
+        url = HOST + "/core/api/coverages/1.0-beta/manager/import/" + \
+            str(collection)+"/jobs"
+        payload = json.dumps({
+            "option": {
+                "type": "internalStorage",
+                "config": {
+                    "type": "file",
+                    "file": {
+                        "stack": True,
+                        "merge": False,
+                        "inputPath": file_import,
+                        "fileName": ""
+                    }
+                },
+                "thumbnail": {
+                    "band": [
+                        1,
+                        2,
+                        3
+                    ]
+                },
+                "mimeType": "tif"
+            }
+        })
+        headers = {
+            'API-Key': Api_Key,
+            'Content-Type': 'application/json'
+        }
+
+        response = requests.request("POST", url, headers=headers, data=payload)
+
+        response_json = response.json()
+        getjobID = response_json['jobID']
+        # print(getjobID)
+        print("start import coverage")
 
         while True:
-            time.sleep(5)
-            url = "https://v2k-dev.vallarismaps.com/core/api/coverages/1.0-beta/manager/export/"+str(collection)+"/jobs/" + str(jobs)
+            time.sleep(10)
+            response_st = requests.request(
+                "GET", f"{HOST}/core/api/coverages/1.0-beta/manager/import/{collection}/jobs/" + getjobID, headers=headers, data=payload)
+            print(
+                f'{str(response_st.json()["status"])} : {str(response_st.json()["progress"])} %')
+            if str(response_st.json()["progress"]) == "100" and response_st.json()["status"] == "succeed":
+                status = response_st.json()["status"]
+                break
+
+            if response_st.json()["status"] == "failed":
+                status = response_st.json()["status"]
+                break
+
+        if os.path.exists(zip_id + ".zip"):
+            os.remove(zip_id + ".zip")
+
+        if status == "succeed":
+            return 'Succeed : ' + HOST + "/management/studio/coverages/" + collection
+
+        else:
+            return 'error to create coverage : failed'
+
+    except Exception as e:
+        if os.path.exists(zip_id + ".zip"):
+            os.remove(zip_id + ".zip")
+
+        print(e)
+        return 'error to create coverage : ' + str(e)
+
+def CreateCoverageTile(parameter, *args, **kwargs):
+    
+    try :
+        reverse = kwargs.get('reverse', 'false')
+        get_class = kwargs.get('color', False)
+
+        collection_in = parameter['dataset_id']
+        collection_out = parameter['dataset_out']
+        style = parameter['style']
+
+        HOST = os.environ["VallarisServer"]
+        Api_Key = os.environ["APIKey"]
+
+        if style == 'gray' :
+
+            url = HOST + "/core/api/coverages/1.0-beta/manager/style/"+str(collection_in)+"/jobs?type=gray"
+
+        elif style == 'rgb' :
+
+            url = HOST + "/core/api/coverages/1.0-beta/manager/style/"+str(collection_in)+"/jobs?type=rgb"
+
+        elif style == 'classification' :
+            
+            if get_class != False :
+                
+                crt_class = len(get_class)
+                
+                min_ = []
+                max_ = []
+                
+                for i in range (0, crt_class):
+                    min_.append(get_class[i]['min'])
+                    max_.append(get_class[i]['max'])
+                    
+                min_get =  min(min_)
+                max_get =  max(max_)
+                
+                total = np.array([min_, max_])
+                mean_get = np.mean(total)
+                stDev_get = np.std(total)
+                
+                option = {
+                        "type": "classification",
+                        "band": 1,
+                        "noData": -999,
+                        "stats": [
+                            {
+                                "band1": {
+                                    "maximum": max_get,
+                                    "mean": mean_get,
+                                    "minimum": min_get,
+                                    "stDev": stDev_get
+                                }
+                            }
+                        ],
+                        "classes": crt_class,
+                        "method": "EqualInterval",
+                        "color": get_class,
+                        "opacity": 100
+                }
+                
+                print(option)
+            
+            else :
+                
+                err = {
+                    "error": 'no class for create style',
+                    "class": [
+                        {
+                            "min": 0.0,
+                            "max": 58.6,
+                            "color": "#08488e"
+                        },
+                        {
+                            "min": 58.6,
+                            "max": 117.2,
+                            "color": "#135fa7"
+                        },
+                        {
+                            "min": 117.2,
+                            "max": 175.8,
+                            "color": "#2676b8"
+                        }
+                    ]
+                }
+                
+                return err
+             
+            # url = HOST + "/core/api/coverages/1.0-beta/manager/style/"+str(collection_in)+"/jobs?type=classification&method=EqualInterval&classes=10&color=viridis&reverse=" + str(reverse)
+
+        else :
+             return 'error to create coverage tile : style not support'
+
+        print("running create coverage tile")
+        
+        if style != 'classification':
 
             payload = ""
             headers = {
-              'API-Key': APIKey
+              'API-Key': Api_Key,
+                'Content-Type': 'application/json'
             }
 
-            response3 = requests.request("GET", url, headers=headers, data=payload)
+            response = requests.request("POST", url, headers=headers, data=payload)
 
-            response4 = response3.json()
+            # print(response)
 
-            
-            try :
-                if response4['code'] == 404 :
-                    res_out = "data permission"
-                    return res_out
-                    break
-            except:
+            print("strat create coverage tile")
 
-                if response4['status'] == 'succeed' or response4['status'] == 'failed':
+            response_json = response.json()
+            # print(response_json)
+        
+            option = response_json['styleDefault']['option']
+        
+        
+        option_style = {
+            "option": option,
+            "collection": {
+                "input": collection_in,
+                "output": collection_out
+            }
+            }
 
-                    if response4['status'] == 'succeed':
+        # print(option_style)
+        url = HOST + "/core/api/coverages/1.0-beta/manager/maketile/"+collection_in+"/jobs"
+        payload = json.dumps(option_style)
+        headers = {
+            'API-Key': Api_Key,
+          'Content-Type': 'application/json'
+        }
 
-                        newpath = str(collection) 
-                        if not os.path.exists(newpath):
-                            os.makedirs(newpath)
+        response = requests.request("POST", url, headers=headers, data=payload)
+        # print(response.text)
 
-                        urllib.request.urlretrieve(response4['file'], newpath + "/"+str(collection)+".zip")
-
-                        with zipfile.ZipFile(newpath + "/" +str(collection) +".zip", 'r') as zip_ref:
-                            zip_ref.extractall(newpath)
-
-                        
-                        mypath = newpath + "/download/"
-                        onlyfiles = [f for f in listdir(mypath) if isfile(join(mypath, f))]
-                
-
-                        with zipfile.ZipFile(newpath + "/download/" +str(onlyfiles[0]), 'r') as zip_ref:
-                            zip_ref.extractall(newpath + "/download")
-                                
-                        name_out = str(onlyfiles[0][0:24])
-
-                        res_out = "succeed : " + newpath + '/download/' + name_out + '/IMAGERY.TIF'
-
-                    else :
-                        res_out = "failed" 
-
-                    break
-                else : 
-                    print("run processing")
-                    res_out = "run processing"
-
-                
-        return res_out
+        job_json = response.json()
+        getjobID =  job_json['jobID']
+        # print(getjobID)
+        print("start import coverage")
     
+        while True :
+            time.sleep(10)
+            response_st = requests.request("GET", f"{HOST}/core/api/coverages/1.0-beta/manager/maketile/{collection_in}/jobs/" + getjobID , headers=headers, data=payload)
+            print(f'{str(response_st.json()["status"])} : {str(response_st.json()["progress"])} %')
+            if str(response_st.json()["progress"]) == "100" and response_st.json()["status"] == "succeed":
+                status = response_st.json()["status"]
+                break
+
+            if response_st.json()["status"] == "failed":
+                status = response_st.json()["status"]
+                break
+        
+        if status == "succeed":
+            return 'Succeed : ' + HOST + "/management/studio/tiles-coverage/" + collection_out
+        
+        else:
+            return 'error to create coverage : failed'
     
     except Exception as e:
-        return 'error :' + str(e)
-            
\ No newline at end of file
+        print(e)
+        return 'error to create coverage : ' + str(e)
\ No newline at end of file
